@@ -1,9 +1,10 @@
 ##++
-##     CGI Lite v3.01
+##     CGI Lite v3.03
 ##
 ##     see separate CHANGES file for detailed history
 ##
-##     Changes in versions 2.03 and newer copyright (c) 2014-2015 Pete Houston
+##     Changes in versions 2.03 and newer copyright
+##     (c) 2014-2015, 2017-2018, 2021 Pete Houston
 ##
 ##     Copyright (c) 1995, 1996, 1997 by Shishir Gundavaram
 ##     All Rights Reserved
@@ -515,7 +516,7 @@ lightweight CGI implementations.
 
 =head1 REPOSITORY
 
-L<https://github.com/openstrike/perl-CGI-Lite>
+L<https://gitlab.com/openstrike/cgi-lite/>
 
 =head1 MAINTAINER
 
@@ -529,23 +530,23 @@ and offering suggestions:
 
 =over 4
 
-=item Eric D. Friedman (friedman@uci.edu)   
+=item Eric D. Friedman
 
-=item Thomas Winzig (tsw@pvo.com)
+=item Thomas Winzig
 
-=item Len Charest (len@cogent.net)
+=item Len Charest
 
-=item Achim Bohnet (ach@rosat.mpe-garching.mpg.de)
+=item Achim Bohnet
 
-=item John E. Townsend (John.E.Townsend@BST.BLS.com)
+=item John E. Townsend
 
-=item Andrew McRae (mcrae@internet.com)
+=item Andrew McRae
 
-=item Dennis Grant (dg50@chrysler.com)
+=item Dennis Grant
 
-=item Scott Neufeld (scott.neufeld@mis.ussurg.com)
+=item Scott Neufeld
 
-=item Raul Almquist (imrs@ShadowMAC.org)
+=item Raul Almquist
 
 =item and many others!
 
@@ -559,7 +560,8 @@ Smylers, Andreas, Ben and Shishir.
 Copyright (c) 1995, 1996, 1997 by Shishir Gundavaram.
 All Rights Reserved.
 
-Changes in versions 2.03 onwards are copyright 2014, 2015 by Pete Houston.
+Changes in versions 2.03 onwards are copyright 2014, 2015, 2017, 2018, 2021
+by Pete Houston.
 
 Permission to use, copy, and  distribute  is  hereby granted,
 providing that the above copyright notice and this permission
@@ -592,7 +594,7 @@ BEGIN {
 	our @EXPORT = qw/browser_escape url_encode url_decode is_dangerous/;
 }
 
-our $VERSION = '3.01';
+our $VERSION = '3.03';
 
 ##++
 ##  Start
@@ -1149,7 +1151,9 @@ sub _parse_multipart_data
 {
 	my ($self, $total_bytes, $boundary) = @_;
 	my $files = {};
-	$boundary = quotemeta ($boundary);
+	my $boundary_re = qr/(.*?)((?:\015?\012)?-*
+		\Q$boundary\E
+		-*[\015\012]*)(?=(.*))/xs;
 
 	eval {
 
@@ -1167,10 +1171,10 @@ sub _parse_multipart_data
 		$platform    = $self->{platform};
 		$eol         = $self->{eol}->{$platform};
 		$directory   = $self->{multipart_dir};
+		$bytes_left  = $total_bytes;
 
-		while (1) {
-			if (   ($byte_count < $total_bytes)
-				&& (length ($current_buffer || '') < ($buffer_size * 2))) {
+		while ($bytes_left) {
+			if ($byte_count < $total_bytes) {
 
 				$bytes_left = $total_bytes - $byte_count;
 				$buffer_size = $bytes_left if ($bytes_left < $buffer_size);
@@ -1204,8 +1208,7 @@ sub _parse_multipart_data
 			##  value that has the first two characters ("--") missing.
 			##--
 
-			if ($current_buffer =~
-				/(.*?)((?:\015?\012)?-*$boundary-*[\015\012]*)(?=(.*))/os) {
+			if ($current_buffer =~ $boundary_re) {
 
 				($store, $this_boundary, $old_data) = ($1, $2, $3);
 
@@ -1284,10 +1287,6 @@ sub _parse_multipart_data
 			} elsif ($old_data) {
 				$store    = $old_data;
 				$old_data = $new_data;
-
-			} else {
-				$store          = $current_buffer;
-				$current_buffer = $new_data;
 			}
 
 			unless ($changed) {
@@ -1313,7 +1312,7 @@ sub _store
 	if ($file) {
 		if ($convert) {
 			if ($platform eq 'PC') {
-				$$info =~ s/\015(?=[^\012])|(?<=[^\015])\012/$eol/og;
+				$$info =~ s/\015(?!\012)|(?<!\015)\012/$eol/og;
 			} else {
 				$$info =~ s/\015\012/$eol/og;
 				$$info =~ s/\015/$eol/og if ($platform ne 'Mac');
